import os
import shutil

import caffe
import numpy as np
import cv2
import sys
from PIL import Image

import time
from sklearn.metrics import jaccard_similarity_score


def processImage(net ,IMAGE,MASKS, outputName='out.jpg'):
	# load image, switch to BGR, subtract mean, and make dims C x H x W for Caffe

	files = os.listdir(IMAGE)
	shutil.rmtree(OUTPUT,ignore_errors=True)
	os.makedirs(OUTPUT)
	scores=[]
	for fn in files:
		img2 = Image.open(IMAGE+fn)
		for imSz in [394]:
			img = img2.resize((imSz,imSz),Image.BICUBIC)
			mask = Image.open(MASKS+fn)
			mask = np.array(mask.resize((imSz,imSz),Image.NEAREST).convert('P'))
			mask = np.where(mask > 0, 1, 0)
			in_ = np.array(img, dtype=np.float32)
			in_ = in_[:, :, ::-1]
			#in_ -= np.array((104.00698793, 116.66876762, 122.67891434))
			in_ = in_.transpose((2, 0, 1))

			#caffe.set_mode_cpu()
			# shape for input (data blob is N x C x H x W), set data
			net.blobs['data'].reshape(1, *in_.shape)
			net.blobs['data'].data[...] = in_
			# run net and take argmax for prediction
			time1 = time.time()
			net.forward()
			time2 = time.time()
			out = net.blobs['score_2classes'].data[0].argmax(axis=0)
			scores.append(jaccard_similarity_score(mask, out))
			out = np.where(out > 0, 255, 0)
			result = Image.fromarray(out.astype(np.uint8))
			out = Image.merge('RGB',[Image.fromarray(out.astype(np.uint8)),Image.fromarray(out.astype(np.uint8)),Image.fromarray(out.astype(np.uint8))])
			res = Image.blend(img,out,0.5)
			res.save(outputName + format(imSz) +'_'+ fn)
			print('function took '+format((time2-time1)*1000)+'ms')
			print('jaccard_similarity_score is: ' + format(np.array(scores)) )
	print('mean jaccard_similarity_score is: ' + format(np.mean(np.array(scores))) )
	return result


if __name__ == '__main__':
	IMAGE = 'images/'
	MASKS = 'masks/'
	OUTPUT = 'caffeResults/'
	# load net

	net = caffe.Net('/home/alp/Workspace/Code/techloq-image-daemon/prototxt_folder/FCN_deploy_SKIN_394.prototxt',
					'/home/alp/Workspace/Data/Models/Skincolor_V_25/snapshot_iter_449800.caffemodel', caffe.TEST)
	image = processImage(net, IMAGE,MASKS,OUTPUT)