import os
import numpy as np
import cv2
import sys
from PIL import Image
import sys, os
sys.path.insert(1, '/usr/src/tensorrt/samples/python/')
import common

import pycuda.driver as cuda
import pycuda.autoinit
import tensorrt as trt
import time
import shutil
from sklearn.metrics import jaccard_similarity_score

TRT_LOGGER = trt.Logger(trt.Logger.INFO)

# Define some global constants about the model which I do not use.
class ModelData(object):
	INPUT_NAME = "data"
	INPUT_SHAPE = (3, 513, 513)
	OUTPUT_SHAPE = (2, 513,513)

	
def build_engine(deploy_file, model_file):
	with trt.Builder(TRT_LOGGER) as builder, builder.create_network() as network, trt.CaffeParser() as parser:
		builder.max_workspace_size = common.GiB(2)

		# Set the parser's plugin factory. Note that we bind the factory to a reference so
		# that we can destroy it later. (parser.plugin_factory_ext is a write-only attribute)
		#parser.plugin_factory_ext = fc_factory

		# Parse the model and build the engine.
		model_tensors = parser.parse(deploy=deploy_file, model=model_file, network=network, dtype=trt.float32)

		network.mark_output(model_tensors.find("score_2classes"))
		return builder.build_cuda_engine(network)

	
def processImage(IMAGE,MASKS, outputName='output.jpg',imSz=394):
	deploy_file = '/Code/prototxt_folder/FCN_deploy_SKIN_394.prototxt'
	model_file='/Models/Skincolor_V_25/snapshot_iter_449800.caffemodel'
	
	files = os.listdir(IMAGE)
	shutil.rmtree(OUTPUT,ignore_errors=True)
	os.makedirs(OUTPUT)
	scores = []
	with build_engine(deploy_file, model_file) as engine:
		inputs, outputs, bindings, stream = common.allocate_buffers(engine)
		with engine.create_execution_context() as context:
			for fn in files:
				img2 = Image.open(IMAGE+fn)
				mask = Image.open(MASKS+fn)
				img2 = img2.resize((imSz,imSz),Image.BICUBIC)
				mask = np.array(mask.resize((imSz,imSz),Image.NEAREST).convert('P'))
				mask = np.where(mask > 0, 1, 0)
				in_ = np.array(img2)
				in_ = in_[:, :, ::-1]
				in_ = in_.transpose((2, 0, 1))
				in_ = np.array(in_, dtype=np.float32)
				img = in_.ravel()
				np.copyto(inputs[0].host, img)
				time1 = time.time()
				[output] = common.do_inference(context, bindings=bindings, inputs=inputs, outputs=outputs, stream=stream)
				time2 = time.time()
				output = np.reshape(output,(2,imSz,imSz))
				out = output.argmax(axis=0)
				scores.append(jaccard_similarity_score(np.array(mask), out))
				out = np.where(out ==1, 255, 0)
				result = Image.fromarray(out.astype(np.uint8))
				out = Image.merge('RGB',[Image.fromarray(out.astype(np.uint8)),Image.fromarray(out.astype(np.uint8)),Image.fromarray(out.astype(np.uint8))])
				res = Image.blend(img2,out,0.5)
				res.save(outputName+fn)
				print('function took '+format((time2-time1)*1000)+'ms')
				print('jaccard_similarity_score is: ' + format(np.array(scores)) )
	print('mean jaccard_similarity_score is: ' + format(np.mean(np.array(scores))) )
	return res

if __name__ == '__main__':
	IMAGE = 'images/'
	MASKS = 'masks/'
	OUTPUT = 'tensorRTResults/'
	# load net
	image = processImage(IMAGE,MASKS,OUTPUT,394)